import {
  Controller,
  Post,
  UploadedFile,
  UseGuards,
  UseInterceptors,
} from '@nestjs/common';
import { ApiBody, ApiConsumes, ApiOperation, ApiTags } from '@nestjs/swagger';
import { MyCardAdminGuard } from '../my-card-admin.guard';
import { FileInterceptor } from '@nestjs/platform-express';
import { FileUploadDto } from '../dto/FileUpload.dto';
import { AppsJson } from '../utility/apps-json-type';
import { AppService } from '../app.service';
import { BlankReturnMessageDto } from '../dto/ReturnMessage.dto';

@Controller('api/admin')
@ApiTags('admin')
@UseGuards(MyCardAdminGuard)
export class AdminController {
  constructor(private readonly appService: AppService) {}

  @Post('migrate')
  @ApiOperation({ summary: '迁移', description: '上传一个 apps.json 文件' })
  @UseInterceptors(FileInterceptor('file'))
  @ApiConsumes('multipart/form-data')
  @ApiBody({
    description: 'apps.json 文件',
    type: FileUploadDto,
  })
  async migrate(
    @UploadedFile('file') file: Express.Multer.File,
  ): Promise<BlankReturnMessageDto> {
    const apps: AppsJson.App[] = JSON.parse(file.buffer.toString());
    return this.appService.migrateFromAppsJson(apps);
  }
}
