import { Connection, IsNull, Not } from 'typeorm';
import { InjectConnection } from '@nestjs/typeorm';
import { Injectable, ConsoleLogger } from '@nestjs/common';
import { AppsJson } from './utility/apps-json-type';
import { App } from './entities/App.entity';
import { BlankReturnMessageDto } from './dto/ReturnMessage.dto';

@Injectable()
export class AppService extends ConsoleLogger {
  constructor(
    @InjectConnection('app')
    private db: Connection,
  ) {
    super('app');
  }

  async getAppsJson() {
    return (
      await this.db
        .getRepository(App)
        .find({ where: { appContent: Not(IsNull()) } })
    ).map((a) => a.app);
  }

  async migrateFromAppsJson(apps: AppsJson.App[]) {
    const targetApps: App[] = [];
    for (const appData of apps) {
      if (!appData.id) {
        throw new BlankReturnMessageDto(
          400,
          `App ${appData.name} is invalid.`,
        ).toException();
      }
      const checkExistingApp = await this.db
        .getRepository(App)
        .findOne({ where: { id: appData.id }, relations: ['history'] });
      if (checkExistingApp) {
        checkExistingApp.updateApp(appData);
        targetApps.push(checkExistingApp);
      } else {
        const app = new App();
        app.id = appData.id;
        app.app = appData;
        targetApps.push(app);
      }
    }
    await this.db.getRepository(App).save(targetApps);
    return new BlankReturnMessageDto(201, 'success');
  }
}
