import { Column, Entity, OneToMany, PrimaryColumn } from 'typeorm';
import { TimeBase } from './TimeBase.entity';
import { AppsJson } from '../utility/apps-json-type';
import { MyCardUser } from '../utility/mycard-auth';
import { AppBase } from './AppBase.entity';
import { AppHistory } from './AppHistory.entity';

@Entity()
export class App extends AppBase {
  @PrimaryColumn('varchar', { length: 64 })
  id: string;

  @Column('varchar', { length: 128, nullable: true })
  authorsStr: string;

  get author(): number[] {
    return this.authorsStr
      ? this.authorsStr.split('|').map((idStr) => parseInt(idStr))
      : [];
  }

  set author(a) {
    this.authorsStr = a.join('|');
  }

  isUserCanEditApp(u: MyCardUser) {
    return u.admin || this.author.includes(u.id);
  }

  @OneToMany(() => AppHistory, (h) => h.parentApp, { cascade: true })
  history: AppHistory[];

  updateApp(appData: AppsJson.App) {
    this.app = appData;
    const h = new AppHistory();
    h.parentApp = this;
    h.time = new Date();
    h.app = appData;
    this.history.push(h);
  }
}
